// #![allow(clippy::print_literal)]
use super::{gdt, interrupts};
use crate::{logger, serial_println};

/// x86_64 initialization
pub fn init() {
    use crate::{network::socket::SimpleSock, relib::network::socket::Socket};

    let mut log_socket_id = SimpleSock::new();
    log_socket_id.register_protocol("Logger".to_string());

    let result = logger::init();
    match result {
        Ok(_) => {
            serial_println!("Logger initialized");
        }
        Err(err) => error!("{}", err),
    }

    gdt::init();

    interrupts::init_idt();
    unsafe { interrupts::PICS.lock().initialize() };
    x86_64::instructions::interrupts::enable();
}
